#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : poll
# @Time         : 2024/5/22 08:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.str_utils.regular_expression import parse_url

from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import openai_response2sse, to_openai_completion_params

from openai import OpenAI, AsyncOpenAI


class Completions(object):
    def __init__(self, api_key=None):
        params = dict(
            api_key=api_key,
            base_url='https://api.chatfire.cn/v1',
            max_retries=1,
        )
        # 如果这一步就报错呢
        self.client = AsyncOpenAI(**params)

    def create(self, request: ChatCompletionRequest):
        logger.debug(str(request.messages))

        prompt = request.messages[-1].get('content')
        data = to_openai_completion_params(request)

        logger.debug(request.messages)

        return OpenAI().chat.completions.create(**data)

    async def acreate(self, request: ChatCompletionRequest, **kwargs):
        logger.debug(str(request.messages))

        prompt = request.messages[-1].get('content')
        data = to_openai_completion_params(request)

        logger.debug(str(request.messages))

        _ = await self.client.chat.completions.create(**data)

        return _



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : speech
# @Time         : 2023/12/26 14:12
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.ai_audio.tts import EdgeTTS
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, status
from fastapi.responses import JSONResponse, StreamingResponse

from chatllm.schemas.openai_types import SpeechCreateRequest  #

router = APIRouter()


@router.post("/audio/speech")
async def create_speech(request: SpeechCreateRequest):
    logger.debug(request)

    # media_types = {
    #     "mp3": "audio/mpeg",
    #     "opus": "audio/opus",
    #     "aac": "audio/aac",
    #     "flac": "audio/flac",
    #     "wav": "audio/wav",
    #     "pcm": "text/event-stream",
    # }
    # media_types.get(request.response_format)

    try:
        data = request.model_dump()
        stream = await EdgeTTS().acreate_for_openai(**data)

        return StreamingResponse(stream, media_type="text/event-stream")

    except Exception as e:
        logger.error(traceback.format_exc())

        return JSONResponse(
            status_code=status.HTTP_400_BAD_REQUEST,
            content={"error": {"message": f"{e}", "type": "api_error"}}
        )


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : file_loader
# @Time         : 2024/3/15 16:48
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json

from meutils.pipe import *
from langchain_community.document_loaders import UnstructuredAPIFileLoader as _UnstructuredAPIFileLoader
from fastapi import APIRouter, File, UploadFile, Query, Form, BackgroundTasks, Depends, HTTPException, Request, status


class MetaData(BaseModel):
    source: Optional[str] = None

    languages: Optional[Any] = []
    page_number: Optional[int] = None
    filename: Optional[str] = None
    filetype: Optional[str] = None
    parent_id: Optional[str] = None
    category: Optional[str] = None

    type: str = 'Document'


class UnstructuredAPIFileLoader(_UnstructuredAPIFileLoader):
    """
    加缓存 异步
    """

    def __init__(
        self,
        file_path: Optional[Union[str, List[str]]] = None,
        mode: Literal["single", "elements", "paged"] = "paged",
        **kwargs
    ):
        super().__init__(file_path, mode=mode, **kwargs)
        base_url = os.getenv("UNSTRUCTURED_BASE_URL", self.url).rstrip('/')
        # url: str = "https://api.unstructured.io/general/v0/general",

        self.url = f"{base_url}/general/v0/general"
        self.api_key = os.getenv("UNSTRUCTURED_API_KEY", "")
        # todo: 重写

    @classmethod
    async def load_for_openai(cls, file: Union[str, UploadFile]):
        """
        from langchain.text_splitter import RecursiveCharacterTextSplitter

        _text_splitter: TextSplitter = RecursiveCharacterTextSplitter()
        _text_splitter.split_documents(docs)

        :return:
        """
        if isinstance(file, str) and file.startswith("http"):
            async with httpx.AsyncClient(timeout=60) as client:
                response = await client.get(url=file)
                file = UploadFile(
                    file=io.BytesIO(response.content),
                    filename=Path(file).name,
                    size=len(response.content),
                    headers=response.headers,  # content_type
                )

        # logger.info(f"file: {file}")

        object = cls(file=file.file, strategy="fast", metadata_filename=file.filename, mode='single')  # todo: 同步转异步
        # object.load_and_split()
        docs = object.load()

        return docs and docs[0].dict() or "{}"  # todo:会不会不止一个文档


if __name__ == '__main__':
    loader = UnstructuredAPIFileLoader(
        # file_path="/Users/betterme/PycharmProjects/AI/ChatLLM/examples/openaisdk/new.pdf",

        # file_path=None,
        file=open("/Users/betterme/PycharmProjects/AI/ChatLLM/data/不存在的骑士01.pdf", 'rb'),
        metadata_filename="new.pdf",
    )

    with timer("解析"):
        print(loader.load_and_split())

    # with timer("解析"):
    #     _ = UnstructuredAPIFileLoader.load_for_openai(
    #         # "https://sfile.chatglm.cn/chatglm4/82834747-0fcf-4ecb-94b0-92e5e749798b.docx"
    #         "https://f.chatllm.vip/test/3MvAKgEoBrBdPgWsQSdTB8.pdf"
    #     )
    #     docs = arun(_)
    #     # print(docs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : yuanbao
# @Time         : 2024/6/11 18:56
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from chatllm.schemas.yuanbao_types import SSEData, BASE_URL, API_CHAT, API_GENERATE_ID, API_DELETE_CONV, GET_AGENT_CHAT
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from meutils.llm.openai_utils import create_chat_completion_chunk, create_chat_completion, token_encoder


# import rev_HunYuan


class Completions(object):
    def __init__(self, api_key: str):
        self.api_key = api_key

        self.httpx_client = httpx.Client(
            base_url=BASE_URL,
            cookies=self.cookies,
            follow_redirects=True,
            timeout=300
        )
        self.httpx_aclient = httpx.AsyncClient(
            base_url=BASE_URL,
            cookies=self.cookies,
            follow_redirects=True,
            timeout=300
        )

    async def acreate(self, request: ChatCompletionRequest):

        history = request.messages[:-1]
        prompt = f"""
        ```json\n{json.dumps(history, ensure_ascii=False, indent=4)}\n```
        ---
        问：{request.last_content}
        """
        # history = request.messages
        # prompt = f"""
        # ```json\n{json.dumps(history, ensure_ascii=False, indent=4)}\n```
        # """

        logger.debug(prompt)

        chunks = self.achat(prompt)

        if request.stream:
            return create_chat_completion_chunk(chunks, redirect_model=request.model)
        else:
            content = ""
            prompt_tokens = 1
            async for chunk in chunks:
                content += chunk
                prompt_tokens += 1
            return create_chat_completion(content, prompt_tokens=prompt_tokens, redirect_model=request.model)

    @cached_property
    def cookies(self):
        hy_user, hy_token, sensorsdata2015jssdkcross = self.api_key.split('|')

        headers = {
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Safari/537.36',
        }

        cookies = {
            'web_uid': f'{uuid.uuid4()}',
            'hy_user': hy_user,
            'hy_token': hy_token,
            'hy_source': 'web',
            'sensorsdata2015jssdkcross': sensorsdata2015jssdkcross,
        }

        return cookies

    def chat(self, prompt: str = '1+1'):
        payload = {
            "prompt": prompt,

            # "model": "gpt_175B_0404",
            # "plugin": "Adaptive",
            # "displayPrompt": "1+1",
            # "displayPromptType": 1,
            # "options": {},
            # "multimedia": [],
            # "agentId": "naQivTmsDa",
            # "version": "v2"
        }
        with self.httpx_client.stream(method="POST", url=f"{API_CHAT}/xxx", json=payload) as response:
            for chunk in response.iter_lines():
                # print(chunk)
                print(SSEData(chunk=chunk, crop_image=False).content)

    async def achat(self, prompt: str = '1+1', chatid: Optional[str] = None):
        payload = {
            "prompt": prompt,

            # "model": "gpt_175B_0404",
            # "plugin": "Adaptive",
            # "displayPrompt": "1+1",
            # "displayPromptType": 1,
            # "options": {},
            # "multimedia": [],
            # "agentId": "naQivTmsDa",
            # "version": "v2"
        }

        chatid = chatid or uuid.uuid4()

        async with self.httpx_aclient.stream(method="POST", url=f"{API_CHAT}/{chatid}", json=payload) as response:
            async for chunk in response.aiter_lines():
                content = SSEData(chunk=chunk, crop_image=True).content
                # print(content)
                yield content

    def generate_id(self, random: bool = True):
        if random:
            return f'{uuid.uuid4()}'
        return self.httpx_client.post(API_GENERATE_ID).text

    def delete_conv(self, chatid):
        response = self.httpx_client.post(f"{API_DELETE_CONV}/{chatid}")
        return response.status_code == 200


# data = '{"model":"gpt_175B_0404","prompt":"1+1","plugin":"Adaptive","displayPrompt":"1+1","displayPromptType":1,"options":{},"multimedia":[],"agentId":"naQivTmsDa","version":"v2"}'

#
# payload = {"model": "gpt_175B_0404", "prompt": "画条可爱的狗", "plugin": "Adaptive", "displayPrompt": "画条可爱的狗",
#            "displayPromptType": 1, "options": {}, "multimedia": [], "agentId": "naQivTmsDa", "version": "v2"}
# # payload = {"model":"gpt_175B_0404","prompt":"搜索下今天的新闻","plugin":"Adaptive"}
#
# chatid = generate_id()
# #
# chatid = 'gtcnTp5C1G'
# chatid = "xx"
# url = f'https://yuanbao.tencent.com/api/chat/{chatid}'
#
# #
# with httpx.stream("POST", url, json=payload, headers=headers, cookies=cookies) as response:
#     for chunk in response.iter_lines():
#         # print(chunk)
#         print(SSEData(chunk=chunk, crop_image=False).content)


if __name__ == '__main__':
    api_key = "bUZenNkB3YaXTbw9|lTb5OHLVMWugG/U9/hzmYdwxKy8t7x2H7yf1iJSmk8ClnXzl3BRouO3LPsdXIAzk|%7B%22distinct_id%22%3A%22100000458739%22%2C%22first_id%22%3A%2218b12e46a2cc31-0422341cefa2bb-18525634-2073600-18b12e46a2d16e0%22%2C%22props%22%3A%7B%22%24latest_traffic_source_type%22%3A%22%E8%87%AA%E7%84%B6%E6%90%9C%E7%B4%A2%E6%B5%81%E9%87%8F%22%2C%22%24latest_utm_medium%22%3A%22cpc%22%2C%22%24search_keyword_id%22%3A%22dfc6144d0063998d000000026667e3ec%22%2C%22%24search_keyword_id_type%22%3A%22baidu_seo_keyword_id%22%2C%22%24search_keyword_id_hash%22%3A5499731361824189%7D%2C%22identities%22%3A%22eyIkaWRlbnRpdHlfY29va2llX2lkIjoiMThiMTJlNDZhMmNjMzEtMDQyMjM0MWNlZmEyYmItMTg1MjU2MzQtMjA3MzYwMC0xOGIxMmU0NmEyZDE2ZTAiLCIkaWRlbnRpdHlfbG9naW5faWQiOiIxMDAwMDA0NTg3MzkifQ%3D%3D%22%2C%22history_login_id%22%3A%7B%22name%22%3A%22%24identity_login_id%22%2C%22value%22%3A%22100000458739%22%7D%2C%22%24device_id%22%3A%2218b12e46a2cc31-0422341cefa2bb-18525634-2073600-18b12e46a2d16e0%22%7D"
    # chatid = generate_id()
    # print(chatid)
    # print(delete_conv(chatid))
    # payload = {
    #     # "model": "gpt_175B_0404",
    #     # "prompt": "1+1",
    #     "prompt": "错了",
    #
    #     # "plugin": "Adaptive",
    #     # "displayPrompt": "1+1",
    #     # "displayPromptType": 1,
    #     # "options": {},
    #     # "multimedia": [],
    #     # "agentId": "naQivTmsDa",
    #     # "version": "v2"
    # }
    # chat(payload)

    async2sync_generator(Completions(api_key).achat('画条狗')) | xprint



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ark_embedding
# @Time         : 2024/5/16 09:25
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


'''
Usage:
DEPRECATED: this is v2 sdk that will be discarded and using v3 instead
1. python3 -m pip install --user volcengine
2. python main.py
'''
import os
from volcengine.maas.v2 import MaasService
from volcengine.maas import MaasException


def test_embeddings(maas, endpoint_id, req):
    try:
        resp = maas.embeddings(endpoint_id, req)
        print(resp)
    except MaasException as e:
        print(e)


if __name__ == '__main__':
    maas = MaasService('maas-api.ml-platform-cn-beijing.volces.com', 'cn-beijing')

    # set ak&sk
    maas.set_ak(os.getenv("ARK_ACCESS_KEY"))
    maas.set_sk(os.getenv("ARK_SECRET_ACCESS_KEY"))

    # embeddings
    embeddingsReq = {
        "input": [
            "天很蓝",
            "海很深"
        ]
    }

    endpoint_id = 'ep-20240516005609-9c9pq'
    test_embeddings(maas, endpoint_id, embeddingsReq)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : deeplx
# @Time         : 2024/4/23 17:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.api.deeplx import translate

from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk as _chat_completion_chunk
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo

from chatllm.utils.openai_utils import token_encoder


class Completions(object):

    def create(self, request: ChatCompletionRequest):
        # logger.debug(str(request.messages))
        # {'code': 200, 'id': 9324310002, 'data': '你好，世界', 'alternatives': ['世界，你好', '你好，世界！', '大家好']}
        # target_lang, text = request.last_content.split(maxsplit=1)[0] # todo: 兼容沉浸式翻译
        #
        # payload = {
        #     "text": text,
        #     "source_lang": "auto",
        #     "target_lang": target_lang,
        #     **self.payload
        # }

        try:

            response = translate(**request.payload)
            chunk = response.get('data')
            # chunk = f"""```json\n{bjson(response)}\n```"""
        except Exception as e:
            chunk = str(e)

            send_message(str(e), title='deeplx')

        if request.stream:
            chat_completion_chunk = _chat_completion_chunk.model_copy(deep=True)
            chat_completion_chunk.choices[0].delta.content = chunk
            yield chat_completion_chunk

            # 结束标识
            chat_completion_chunk.choices[0].delta.content = ""
            chat_completion_chunk.choices[0].finish_reason = "stop"
            yield chat_completion_chunk

        else:
            chat_completion.usage.prompt_tokens = len(token_encoder.encode(request.payload.get('text')))
            chat_completion.usage.completion_tokens = 1
            chat_completion.choices[0].message.content = chunk
            yield chat_completion

    async def acreate(self, request: ChatCompletionRequest, **kwargs):
        pass


if __name__ == '__main__':
    payload = {
        "text": '火哥AI是最棒的',
        "source_lang": 'auto',
        "target_lang": 'EN'
    }

    request = ChatCompletionRequest(model='deeplx', payload=payload, stream=False)
    response = Completions().create(request)
    print(json.loads(list(response)[0].choices[0].message.content))

    for i in Completions().create(request):
        print(i)

    # import tiktoken
    #
    # print(tiktoken.get_encoding('cl100k_base').encode('我是谁'))

    import openai

    openai.OpenAI.chat.completions.create()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatglm_web
# @Time         : 2024/3/11 18:52
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm 🧩 🔨[🔨](…….)
# @Description  : https://github.com/ikechan8370/chatgpt-plugin/blob/32af7b9a74fdfbd329f5977c6e3fb5b3928ed0f1/client/ChatGLM4Client.js#L6
import shortuuid

from meutils.pipe import *
from meutils.notice.feishu import send_message

from chatllm.schemas import chatglm_types
from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest
from chatllm.utils.openai_utils import openai_response2sse
from fastapi import UploadFile
from openai.types.file_object import FileObject


class Completions(object):
    def __init__(self, **client_params):
        self.api_key = client_params.get('api_key')
        self.access_token = self.get_access_token(self.api_key)

        self.httpx_client = httpx.Client(headers=self.headers, follow_redirects=True)
        self.httpx_aclient = httpx.AsyncClient(headers=self.headers, follow_redirects=True)

    # def create(self):
    #
    #     self.httpx_client.post(url, json={})

    def create(self, request: ChatCompletionRequest):
        request = self.do_request(request)

        url = "https://chatglm.cn/chatglm/backend-api/assistant/stream"
        payload = isinstance(request, dict) and request or request.model_dump()
        # response = self.httpx_client.post(url=url, json=payload)
        response: httpx.Response
        with self.httpx_client.stream("POST", url=url, json=payload, timeout=200) as response:

            content = ""
            for chunk in response.iter_lines():
                for chat_completion_chunk in self.do_chunk(chunk):
                    _ = chat_completion_chunk.choices[0].delta.content
                    chat_completion_chunk.choices[0].delta.content = _.split(content)[-1] if content else _
                    yield chat_completion_chunk
                    content = _

    async def acreate(self, request: Union[dict, ChatCompletionRequest]):
        request = self.do_request(request)

        url = "https://chatglm.cn/chatglm/backend-api/assistant/stream"
        payload = isinstance(request, dict) and request or request.model_dump()
        # response = self.httpx_client.post(url=url, json=payload)
        response: httpx.Response
        async with self.httpx_aclient.stream("POST", url=url, json=payload, timeout=200) as response:
            content = ""
            async for chunk in response.aiter_lines():
                for chat_completion_chunk in self.do_chunk(chunk):
                    _ = chat_completion_chunk.choices[0].delta.content
                    chat_completion_chunk.choices[0].delta.content = _.split(content)[-1] if content else _
                    yield chat_completion_chunk
                    content = _

    def create_sse(self, request: ChatCompletionRequest):
        if request.stream:
            return openai_response2sse(self.acreate(request), redirect_model=request.model)
        else:
            _chat_completion = chat_completion.model_copy(deep=True)
            _chat_completion.usage.prompt_tokens = len(request.messages[-1]["content"])
            for i in self.create(request):  # todo: 异步
                _chat_completion.choices[0].message.content += i.choices[0].delta.content

            return openai_response2sse(_chat_completion, redirect_model=request.model)

    def do_chunk(self, chunk):

        if chunk := chunk.strip().strip("event:message\ndata: ").strip():

            # logger.debug(chunk)

            chunk = chatglm_types.Data.model_validate_json(chunk)
            content = chunk.parts and chunk.parts[0].markdown_data
            if content:
                # logger.debug(content)
                # yield content
                chat_completion_chunk.choices[0].delta.content = content
                yield chat_completion_chunk

            if chunk.status == 'finish':
                _ = chat_completion_chunk.model_copy(deep=True)
                _.choices[0].delta.content = ""
                _.choices[0].finish_reason = "stop"  # 特殊
                yield _
                return

    def do_request(self, request: ChatCompletionRequest):

        history = request.messages[:-1]
        history = history and f"""参考系统历史对话，以聊天的语气回答问题：\n```json\n{json.dumps(history, ensure_ascii=False, indent=4)}\n```"""  # todo: 优化

        question = request.messages[-1]["content"]
        if isinstance(question, list):  # todo: 兼容多模态
            # 文件  {"type": "image_url", "image_url": {"url": image_url1}}
            # {
            #     "type": "http://ai.chatfire.cn/files/document/绩效面谈表-模版-nine-1710139239100-nine-66b3829d5.pdf text"
            # }

            send_message(request.messages)

        request.messages = [
            {
                'role': 'user',
                'content': [
                    {
                        'type': 'text',
                        'text': f"""{history}\n\n问题：{question}""" if history else question
                    }
                ]
            }
        ]

        # logger.debug(request)

        return request

    def file_extract(self):
        """
        {
          "message": "success",
          "result": {
            "file_id": "chatglm4/f960da34-0041-4662-96e7-791ea455618a.png",
            "file_name": "8E8F00E40FED35D93339FF66691017CC.png",
            "file_size": 0,
            "file_url": "https://sfile.chatglm.cn/chatglm4/f960da34-0041-4662-96e7-791ea455618a.png",
            "height": 1024,
            "width": 1024
          },
          "status": 0
        }
        :return:
        """
        url = "https://chatglm.cn/chatglm/backend-api/assistant/file_upload"
        pass

    @property
    def headers(self):
        return {
            'Authorization': f"Bearer {self.access_token}",
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36'
        }

    @staticmethod
    @ttl_cache(3600 * 2)
    def get_access_token(refresh_token=None):  # 设计重试
        refresh_token = refresh_token or "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJmcmVzaCI6ZmFsc2UsImlhdCI6MTcxMDE0OTU3OCwianRpIjoiYmQ3YWI2ZDItNWViNS00YmVmLTlmMWMtYzU5NTMwM2IyN2ZkIiwidHlwZSI6InJlZnJlc2giLCJzdWIiOiIzNmE4NmM1Yzc2Y2Q0MTcyYTE5NGYxMjQwZTgyMmIwOSIsIm5iZiI6MTcxMDE0OTU3OCwiZXhwIjoxNzI1NzAxNTc4LCJ1aWQiOiI2NDRhM2QwY2JhMjU4NWU5MDQ2MDM5ZGIiLCJ1cGxhdGZvcm0iOiIiLCJyb2xlcyI6WyJ1bmF1dGhlZF91c2VyIl19.gN8ci_OO8Pp0t3wZ3v1lG2X1xoLgGushf3fkm5pRl0M"

        headers = {
            'Authorization': f"Bearer {refresh_token}",
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36'
        }
        url = "https://chatglm.cn/chatglm/backend-api/v1/user/refresh"
        response = httpx.post(url, headers=headers)

        # logger.debug(refresh_token)
        # logger.debug(response.text)
        # logger.debug(response.status_code)

        if response.status_code != 200:
            send_message(f"GLM refresh_token:\n\n{response.text}\n\n{refresh_token}", title="GLM")
            response.raise_for_status()

        # refresh_token = response.get("refresh_token") # 是否去更新
        return response.json().get("result", {}).get("accessToken")

    async def put_object_for_openai(
        self,
        file: Union[bytes, str, Path, UploadFile],
        purpose: str = "chatpdf_file_upload",  # file_upload
        filename: Optional[str] = None,
    ):  # filename => file url 预览https://sfile.chatglm.cn/doc_mobile/fdfeeea2-2721-445a-a8f6-b70a7972c1da.pdf
        if isinstance(file, bytes):
            filename = filename or f"{shortuuid.random()}.png"

        elif isinstance(file, (str, Path)) and Path(file).exists():
            filename = Path(file).name
            file = Path(file).read_bytes()
        else:
            filename = file.filename or file.file.name
            file = file.file

        # base_url = "https://chatglm.cn/chatglm/backend-api/v1/chatpdf_file_upload"
        #
        # base_url = "https://chatglm.cn/chatglm/backend-api/assistant/file_upload"

        # try:
        # 通用
        base_url = "https://chatglm.cn/chatglm/backend-api"
        url = "/v1/chatpdf_file_upload" if purpose == "chatpdf_file_upload" else "/assistant/file_upload"

        payload = {"file": (filename, file)}
        try:
            async with httpx.AsyncClient(headers=self.headers, base_url=base_url, timeout=60) as client:
                response = await client.post(url=url, files=payload)
                if response.status_code != 200:
                    response.raise_for_status()

                # logger.debug(response.json())
                # {'message': '内容已超过对话长度62.25%，请删减部分文件', 'result': [], 'status': 2}

                result = response.json().get("result") or {}

                return FileObject.construct(
                    id=result.get("file_id"),
                    bytes=result.get("content_length"),
                    created_at=int(time.time()),
                    filename=result.get("file_url"),  # result.get("file_name")
                    object='file',
                    purpose=purpose,
                    status='processed' if result else "error",
                    status_details=result
                )

        except Exception as e:
            return str(e)  # todo


# {
#     "message": "success",
#     "result": {
#         "content_length": 37910,
#         "content_raw_length": 37910,
#         "conversation_id": "65f43c951f86227d538333d9",
#         "file_url": "https://sfile.chatglm.cn/doc_mobile/fdfeeea2-2721-445a-a8f6-b70a7972c1da.pdf",
#         "prompt": "东北证券股份有限公司合规手册（东证合规发〔2023〕22号 20231228）.pdf",
#         "questions": [
#             "东北证券合规手册的主要内容包括哪些章节？",
#             "东北证券合规手册对工作人员有哪些具体要求？",
#             "东北证券合规手册对各类业务有哪些合规底线要求？"
#         ],
#         "severity_type": 0,
#         "summary": "这份合规手册来自东北证券股份有限公司，旨在进一步落实“合规、诚信、专业、稳健”的行业文化要求，规范公司各单位及工作人员执业行为，有效防范合规风险，促进全体工作人员形成“全员合规、合规从管理层做起、合规创造价值、合规是公司生存基础”的合规理念和“十个坚持、十个反对”的荣辱观。合规手册包括合规管理、工作人员执业行为规范、公司各类业务的合规底线以及需要遵循的其他规定等部分。此外，该手册还强调了公司合规管理的目标、原则和基本要求，包括充分了解客户信息、合理划分客户类别和产品风险等级、持续督促客户规范证券发行行为等。同时，该手册还明确规定了各类业务的合规底线，包括证券经纪业务、投资银行业务、证券自营及衍生品业务等。"
#     },
#     "status": 0
# }

# {
#     "message": "success",
#     "result": {
#         "content_length": 17908,
#         "content_raw_length": 17908,
#         "file_id": "chatglm4/82834747-0fcf-4ecb-94b0-92e5e749798b.docx",
#         "file_name": "中华人民共和国公司法（2023年修订 20231229 20240701）.docx",
#         "file_size": 0,
#         "file_url": "https://sfile.chatglm.cn/chatglm4/82834747-0fcf-4ecb-94b0-92e5e749798b.docx",
#         "questions": [],
#         "questions_usage": {},
#         "summary": "",
#         "summary_usage": {}
#     },
#     "status": 0
# }

if __name__ == '__main__':
    # print(Completions.get_access_token())

    # data = {
    #     "assistant_id": "65940acff94777010aa6b796",
    #     "conversation_id": "",
    #     # "conversation_id": "",
    #     # "meta_data": {
    #     #     "is_test": False,
    #     #     "input_question_type": "xxxx",
    #     #     "channel": "",
    #     #     "draft_id": ""
    #     # },
    #     "messages": [
    #         {
    #             "role": "user",
    #             "content": [
    #                 {
    #                     "type": "text",
    #                     # "text": "你是一个资深民商事律师，请你运用联网功能，帮我解决以下问题：\n压岁钱的所有权归谁，父母是否有权支配孩子压岁钱？\n请帮我写出相关法条和判例。",
    #                     # "text": "南京今天天气"
    #                     # "text": "画只猫"
    #                     # "text": "1+2"
    #                     "text": "总结一下 http://www.weather.com.cn/weather/101190101.shtml"
    #
    #                 }
    #             ]
    #         }
    #
    #     ]
    # }

    # data = {
    #     # "assistant_id": "65940acff94777010aa6b796",
    #     "assistant_id": "65f41c8a9c0ebbcbe28bb9c1",  # 智能体
    #     # "conversation_id": "65f41fe143c80c1f80b55150",
    #     # "conversation_id": "",
    #     # "meta_data": {
    #     #     "is_test": False,
    #     #     "input_question_type": "xxxx",
    #     #     "channel": "",
    #     #     "draft_id": ""
    #     # },
    #     "messages": [
    #         {
    #             "role": "user",
    #             "content": "从业人员可以炒股吗"
    #         }
    #
    #     ]
    # }
    #
    # for i in Completions().create(ChatCompletionRequest(**data)):
    #     print(i.choices[0].delta.content, end="")
    #     # pass

    _ = Completions().put_object_for_openai(Path("/Users/betterme/PycharmProjects/AI/QR.png").read_bytes(), "")  # 上传文件

    print(arun(_, debug=True))
    print(_)
    pass
